/* valaccodestruct.c generated by valac, the Vala compiler
 * generated from valaccodestruct.vala, do not modify */

/* valaccodestruct.vala
 *
 * Copyright (C) 2006-2010  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include "valaccode.h"
#include <stdlib.h>
#include <string.h>
#include <valagee.h>

#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_iterable_unref0(var) ((var == NULL) ? NULL : (var = (vala_iterable_unref (var), NULL)))
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))

struct _ValaCCodeStructPrivate {
	gchar* _name;
	ValaList* declarations;
};


static gpointer vala_ccode_struct_parent_class = NULL;

#define VALA_CCODE_STRUCT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_STRUCT, ValaCCodeStructPrivate))
static void vala_ccode_struct_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
static void vala_ccode_struct_finalize (ValaCCodeNode * obj);


ValaCCodeStruct* vala_ccode_struct_construct (GType object_type, const gchar* name) {
	ValaCCodeStruct* self = NULL;
	const gchar* _tmp0_;
	g_return_val_if_fail (name != NULL, NULL);
	self = (ValaCCodeStruct*) vala_ccode_node_construct (object_type);
	_tmp0_ = name;
	vala_ccode_struct_set_name (self, _tmp0_);
	return self;
}


ValaCCodeStruct* vala_ccode_struct_new (const gchar* name) {
	return vala_ccode_struct_construct (VALA_TYPE_CCODE_STRUCT, name);
}


/**
 * Adds the specified declaration as member to this struct.
 *
 * @param decl a variable declaration
 */
void vala_ccode_struct_add_declaration (ValaCCodeStruct* self, ValaCCodeDeclaration* decl) {
	ValaList* _tmp0_;
	ValaCCodeDeclaration* _tmp1_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (decl != NULL);
	_tmp0_ = self->priv->declarations;
	_tmp1_ = decl;
	vala_collection_add ((ValaCollection*) _tmp0_, _tmp1_);
}


/**
 * Adds a variable with the specified type and name to this struct.
 *
 * @param type_name field type
 * @param name      member name
 */
void vala_ccode_struct_add_field (ValaCCodeStruct* self, const gchar* type_name, const gchar* name, ValaCCodeModifiers modifiers, ValaCCodeDeclaratorSuffix* declarator_suffix) {
	ValaCCodeDeclaration* decl = NULL;
	const gchar* _tmp0_;
	ValaCCodeDeclaration* _tmp1_;
	const gchar* _tmp2_;
	ValaCCodeDeclaratorSuffix* _tmp3_;
	ValaCCodeVariableDeclarator* _tmp4_;
	ValaCCodeVariableDeclarator* _tmp5_;
	ValaCCodeModifiers _tmp6_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (type_name != NULL);
	g_return_if_fail (name != NULL);
	_tmp0_ = type_name;
	_tmp1_ = vala_ccode_declaration_new (_tmp0_);
	decl = _tmp1_;
	_tmp2_ = name;
	_tmp3_ = declarator_suffix;
	_tmp4_ = vala_ccode_variable_declarator_new (_tmp2_, NULL, _tmp3_);
	_tmp5_ = _tmp4_;
	vala_ccode_declaration_add_declarator (decl, (ValaCCodeDeclarator*) _tmp5_);
	_vala_ccode_node_unref0 (_tmp5_);
	_tmp6_ = modifiers;
	vala_ccode_node_set_modifiers ((ValaCCodeNode*) decl, _tmp6_);
	vala_ccode_struct_add_declaration (self, decl);
	_vala_ccode_node_unref0 (decl);
}


static gpointer _vala_iterable_ref0 (gpointer self) {
	return self ? vala_iterable_ref (self) : NULL;
}


static void vala_ccode_struct_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeStruct * self;
	ValaCCodeWriter* _tmp0_;
	ValaCCodeWriter* _tmp1_;
	const gchar* _tmp2_;
	ValaCCodeWriter* _tmp3_;
	ValaCCodeWriter* _tmp17_;
	ValaCCodeModifiers _tmp18_;
	ValaCCodeModifiers _tmp19_;
	ValaCCodeWriter* _tmp21_;
	ValaCCodeWriter* _tmp22_;
	ValaCCodeWriter* _tmp23_;
	self = (ValaCCodeStruct*) base;
	g_return_if_fail (writer != NULL);
	_tmp0_ = writer;
	vala_ccode_writer_write_string (_tmp0_, "struct ");
	_tmp1_ = writer;
	_tmp2_ = self->priv->_name;
	vala_ccode_writer_write_string (_tmp1_, _tmp2_);
	_tmp3_ = writer;
	vala_ccode_writer_write_begin_block (_tmp3_);
	{
		ValaList* _decl_list = NULL;
		ValaList* _tmp4_;
		ValaList* _tmp5_;
		gint _decl_size = 0;
		ValaList* _tmp6_;
		gint _tmp7_;
		gint _tmp8_;
		gint _decl_index = 0;
		_tmp4_ = self->priv->declarations;
		_tmp5_ = _vala_iterable_ref0 (_tmp4_);
		_decl_list = _tmp5_;
		_tmp6_ = _decl_list;
		_tmp7_ = vala_collection_get_size ((ValaCollection*) _tmp6_);
		_tmp8_ = _tmp7_;
		_decl_size = _tmp8_;
		_decl_index = -1;
		while (TRUE) {
			gint _tmp9_;
			gint _tmp10_;
			gint _tmp11_;
			ValaCCodeDeclaration* decl = NULL;
			ValaList* _tmp12_;
			gint _tmp13_;
			gpointer _tmp14_;
			ValaCCodeDeclaration* _tmp15_;
			ValaCCodeWriter* _tmp16_;
			_tmp9_ = _decl_index;
			_decl_index = _tmp9_ + 1;
			_tmp10_ = _decl_index;
			_tmp11_ = _decl_size;
			if (!(_tmp10_ < _tmp11_)) {
				break;
			}
			_tmp12_ = _decl_list;
			_tmp13_ = _decl_index;
			_tmp14_ = vala_list_get (_tmp12_, _tmp13_);
			decl = (ValaCCodeDeclaration*) _tmp14_;
			_tmp15_ = decl;
			_tmp16_ = writer;
			vala_ccode_node_write_declaration ((ValaCCodeNode*) _tmp15_, _tmp16_);
			_vala_ccode_node_unref0 (decl);
		}
		_vala_iterable_unref0 (_decl_list);
	}
	_tmp17_ = writer;
	vala_ccode_writer_write_end_block (_tmp17_);
	_tmp18_ = vala_ccode_node_get_modifiers ((ValaCCodeNode*) self);
	_tmp19_ = _tmp18_;
	if ((_tmp19_ & VALA_CCODE_MODIFIERS_DEPRECATED) == VALA_CCODE_MODIFIERS_DEPRECATED) {
		ValaCCodeWriter* _tmp20_;
		_tmp20_ = writer;
		vala_ccode_writer_write_string (_tmp20_, " G_GNUC_DEPRECATED");
	}
	_tmp21_ = writer;
	vala_ccode_writer_write_string (_tmp21_, ";");
	_tmp22_ = writer;
	vala_ccode_writer_write_newline (_tmp22_);
	_tmp23_ = writer;
	vala_ccode_writer_write_newline (_tmp23_);
}


const gchar* vala_ccode_struct_get_name (ValaCCodeStruct* self) {
	const gchar* result;
	const gchar* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = self->priv->_name;
	result = _tmp0_;
	return result;
}


void vala_ccode_struct_set_name (ValaCCodeStruct* self, const gchar* value) {
	const gchar* _tmp0_;
	gchar* _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = value;
	_tmp1_ = g_strdup (_tmp0_);
	_g_free0 (self->priv->_name);
	self->priv->_name = _tmp1_;
}


gboolean vala_ccode_struct_get_is_empty (ValaCCodeStruct* self) {
	gboolean result;
	ValaList* _tmp0_;
	gint _tmp1_;
	gint _tmp2_;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0_ = self->priv->declarations;
	_tmp1_ = vala_collection_get_size ((ValaCollection*) _tmp0_);
	_tmp2_ = _tmp1_;
	result = _tmp2_ == 0;
	return result;
}


static void vala_ccode_struct_class_init (ValaCCodeStructClass * klass) {
	vala_ccode_struct_parent_class = g_type_class_peek_parent (klass);
	((ValaCCodeNodeClass *) klass)->finalize = vala_ccode_struct_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeStructPrivate));
	((ValaCCodeNodeClass *) klass)->write = (void (*) (ValaCCodeNode *, ValaCCodeWriter*)) vala_ccode_struct_real_write;
}


static void vala_ccode_struct_instance_init (ValaCCodeStruct * self) {
	GEqualFunc _tmp0_;
	ValaArrayList* _tmp1_;
	self->priv = VALA_CCODE_STRUCT_GET_PRIVATE (self);
	_tmp0_ = g_direct_equal;
	_tmp1_ = vala_array_list_new (VALA_TYPE_CCODE_DECLARATION, (GBoxedCopyFunc) vala_ccode_node_ref, (GDestroyNotify) vala_ccode_node_unref, _tmp0_);
	self->priv->declarations = (ValaList*) _tmp1_;
}


static void vala_ccode_struct_finalize (ValaCCodeNode * obj) {
	ValaCCodeStruct * self;
	self = G_TYPE_CHECK_INSTANCE_CAST (obj, VALA_TYPE_CCODE_STRUCT, ValaCCodeStruct);
	_g_free0 (self->priv->_name);
	_vala_iterable_unref0 (self->priv->declarations);
	VALA_CCODE_NODE_CLASS (vala_ccode_struct_parent_class)->finalize (obj);
}


/**
 * Represents a struct declaration in the C code.
 */
GType vala_ccode_struct_get_type (void) {
	static volatile gsize vala_ccode_struct_type_id__volatile = 0;
	if (g_once_init_enter (&vala_ccode_struct_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeStructClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_struct_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeStruct), 0, (GInstanceInitFunc) vala_ccode_struct_instance_init, NULL };
		GType vala_ccode_struct_type_id;
		vala_ccode_struct_type_id = g_type_register_static (VALA_TYPE_CCODE_NODE, "ValaCCodeStruct", &g_define_type_info, 0);
		g_once_init_leave (&vala_ccode_struct_type_id__volatile, vala_ccode_struct_type_id);
	}
	return vala_ccode_struct_type_id__volatile;
}



